#!/usr/bin/ruby
# -*- coding: utf-8 -*-

require "rubygems"
require "prawn"
require "prawn/layout"

# I want to use cm2pt() everywhere.
include Prawn::Measurements

module Prawn
  class Document
    alias_method :orig_start_new_page, :start_new_page unless method_defined? :orig_start_new_page

    def run_before_new_page(&block)
      @run_before_new_page_blocks ||= Array.new
      @run_before_new_page_blocks <<  Proc.new(&block)
    end

    def run_after_new_page(&block)
      @run_after_new_page_blocks ||= Array.new
      @run_after_new_page_blocks <<  Proc.new(&block)
    end

    def start_new_page(options = {})
      run_blocks @run_before_new_page_blocks, options
      self.orig_start_new_page options
      run_blocks @run_after_new_page_blocks
    end

    protected

    def run_blocks(blocks, *args)
      return if !blocks || blocks.empty?
      blocks.each { |block| block.arity == 0 ? self.instance_eval(&block) : block.call(self, *args) }
    end
  end
end

a4_paper          =  {
  :page_size      => "A4",
  :top_margin     => cm2pt(9.2),
  :left_margin    => cm2pt(2.5),
  :bottom_margin  => cm2pt(3.5),
  :right_margin   => cm2pt(2.5),
}

Prawn::Document.generate "start_new_page_demo.pdf", a4_paper do
  run_before_new_page do |pdf, options|
    options[:top_margin] = cm2pt(4.0)
  end

  run_after_new_page do
    image "background_page2_200dpi.png",
      :at  => [-bounds.absolute_left, Prawn::Document::SIZES["A4"][1] - bounds.absolute_bottom],
      :fit => Prawn::Document::SIZES["A4"]
  end

  # This image only appears on the first page.
  image "background_page1_200dpi.png",
    :at  => [-bounds.absolute_left, Prawn::Document::SIZES["A4"][1] - bounds.absolute_bottom],
    :fit => Prawn::Document::SIZES["A4"]

  # Draw the address box on the first page.
  canvas do
    bounding_box([cm2pt(2.5), cm2pt(24)], :width => cm2pt(8), :height => cm2pt(3.4)) do
      address  = "Musterfirma GmbH\n\n"
      address += "z. Hd. Gaby Musterfra\n"
      address += "Beispielsstraße 123\n"
      address += "54321 Irgendwohausen\n"
      address += "Deutschland\n"

      text address
    end
  end

  # Draw enough text to ensure that a second page is created.
  text "Hello, page 1!\n\n"
  text File.read("lorem_ipsum.txt") * 2
  text "\nAnd now we're on page 2."
end
